(*******************************************************************
 *
 *      Component: TRpnButton v1.0
 *         Author: Richard Philip Nguyen (rpngu3@hotbot.com)
 *           Date: 31 January 1999
 *      CopyRight: Richard Philip Nguyen
 *         Status: Freeware
 *
 *    Description: Nice button component that mimics the behaviour of
 *                 buttons found in some homepages
 *
 *          Legal: You are free to use this component in personal and
 *                 commercial applications as long as due credit is given.
 *
 *                 If you change the source code, I ask you to send me
 *                 a copy of your modified code so I can update the
 *                 the component for the next release.
 *
 *        Histoty: 1 Feb 1999 - First release of the component
 *
 *******************************************************************)

{$R RpnButton.dcr}

unit RpnButton;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

type
  TRpnButton = class(TCustomControl)
  private
    { Private declarations }
    FCaption          : TCaption;
    FNormalColor      : TColor;
    FHighlightedColor : TColor;
    Highlighted       : boolean;
    Pressed           : boolean;

    procedure SetCaption(value: TCaption);
    procedure SetNormalColor(value: TColor);
    procedure SetHighlightedColor(value: TColor);
    procedure DrawOutline(Highlighted:boolean);
    procedure CMMouseEnter(var msg: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave(var msg: TMessage); message CM_MOUSELEAVE;
    procedure WMLButtonDown(var msg: TWMLButtonDown); message WM_LBUTTONDOWN;
    procedure WMLButtonUp(var msg: TWMLButtonUp); message WM_LBUTTONUP;
  protected
    { Protected declarations }
    procedure Paint; override;
  public
    { Public declarations }
    constructor Create(AOwner : TComponent); override;
  published
    { Published declarations }
    property Caption read FCaption write SetCaption stored True;
    property Font;
    property NormalColor      : TColor read FNormalColor      write SetNormalColor;
    property HighlightedColor : TColor read FHighlightedColor write SetHighlightedColor;
    property Visible;
    property OnClick;
    property ShowHint;
  end;

procedure Register;

implementation

  constructor TRpnButton.Create(AOwner : TComponent);
  (*************************************************)
  begin
    Inherited CReate(AOwner);

    ControlStyle      := [csClickEvents, csCaptureMouse, csOpaque];

    Caption           := 'RpnButton';
    Width             := 105;
    Height            := 25;
    Highlighted       := False;
    Pressed           := false;
    Font.Color        := clWhite;
    Font.Size         := 10;
    Font.Style        := [fsBold];
    Visible           := True;
    FHighlightedColor := $004D4D4D;
    FNormalColor      := $00A3A3A3;
    ShowHint          := False;
  end;

  procedure TRpnButton.SetCaption(value: TCaption);
  (***********************************************)
  begin
    FCaption := value;
    RePaint();
  end;

  procedure TRpnButton.SetNormalColor(value: TColor);
  (*************************************************)
  begin
    FNormalColor := value;
    RePaint();
  end;

  procedure TRpnButton.SetHighlightedColor(value: TColor);
  (******************************************************)
  begin
    FHighlightedColor := value;
    RePaint();
  end;

  procedure TRpnButton.DrawOutline(Highlighted:boolean);
  (****************************************************)
  begin
    Canvas.Pen.Width := 3;

    if Highlighted then begin
      if Pressed then
        Canvas.Pen.Color := clBlack
      else
        Canvas.Pen.Color := clGray
    end
    else
      Canvas.Pen.Color := clSilver;

    Canvas.MoveTo(0, Height-1);
    Canvas.LineTo(0, 0);
    Canvas.LineTo(Width-1, 0);

    if Highlighted then begin
      if Pressed then
        Canvas.Pen.Color := clGray
      else
        Canvas.Pen.Color := clBlack
    end
    else
      Canvas.Pen.Color := clGray;

    Canvas.LineTo(Width-1, Height-1);
    Canvas.LineTo(0, Height-1);
  end;

  procedure TRpnButton.Paint;
  (*************************)
  var
    tsSize : TSize;
    cx, cy : longint;
  begin
    if not Visible then exit;

    DrawOutline(Highlighted);

    // fills canvas
    if Highlighted then
      Canvas.Brush.Color := FHighlightedColor
    else
      Canvas.Brush.Color := FNormalColor;

    Canvas.FillRect(Rect(2, 2, Width-2, Height-2));

    // draws caption
    if Caption<>'' then begin
      Canvas.Font := Self.Font;
      GetTextExtentPoint(Canvas.handle,@Caption[1],Length(Caption),tsSize);
      cx := ( Width - tsSize.cx) div 2;
      cy := (Height - tsSize.cy) div 2;
      if Pressed then begin
        Inc(cx);
        Inc(cy);
      end;

      Canvas.TextOut(cx, cy, Caption);
    end;
  end;

  procedure TRpnButton.CMMouseEnter(var msg: TMessage);
  (****************************************************)
  begin
    if Visible and (not Highlighted)then begin
      Highlighted:=True;
      Repaint();
    end;
  end;

  procedure TRpnButton.CMMouseLeave(var msg: TMessage);
  (***************************************************)
  begin
    if Visible and Highlighted then begin
      Highlighted:=False;
      Repaint();
    end;
  end;

  procedure TRpnButton.WMLButtonDown(var msg: TWMLButtonDown);
  (**********************************************************)
  begin
    if not Pressed then begin
      Pressed:=True;
      Repaint();
      Click;
    end;
  end;

  procedure TRpnButton.WMLButtonUp(var msg: TWMLButtonUp);
  (******************************************************)
  begin
    if Pressed then begin
      Pressed:=False;
      Repaint();
    end;
  end;

  procedure Register;
  (*****************)
  begin
    RegisterComponents('Rpn', [TRpnButton]);
  end;

end.
